@@ -48,6 +48,12 @@
 #define UNUSED __attribute__ ((unused))
 #define UNUSED_DATA UNUSED gconstpointer data
 
+/* avoid leak reports inside assertions; leaking stuff on assertion failures does not matter in tests */
+#if !defined(__clang__)
+#pragma GCC diagnostic ignored "-Wanalyzer-malloc-leak"
+#pragma GCC diagnostic ignored "-Wanalyzer-file-leak"
+#endif
+
 static gboolean has_real_sysfs;
 
 static int slow_testbed_factor = 1;
@@ -571,40 +577,6 @@ t_testbed_set_property(UMockdevTestbedFixture * fixture, UNUSED_DATA)
     g_object_unref(device);
 }
 
-struct event_counter {
-    unsigned add;
-    unsigned remove;
-    unsigned change;
-    gchar last_device[1024];
-};
-
-static void
-on_uevent(UNUSED GUdevClient *client, const gchar *action, GUdevDevice *device, gpointer user_data)
-{
-    struct event_counter *counter = (struct event_counter *)user_data;
-
-    g_debug("on_uevent action %s device %s", action, g_udev_device_get_sysfs_path(device));
-
-    if (strcmp(action, "add") == 0)
-	counter->add++;
-    else if (strcmp(action, "remove") == 0)
-	counter->remove++;
-    else if (strcmp(action, "change") == 0)
-	counter->change++;
-    else
-	g_assert_not_reached();
-
-    strncpy(counter->last_device, g_udev_device_get_sysfs_path(device), sizeof(counter->last_device) - 1);
-}
-
-static gboolean
-on_timeout(gpointer user_data)
-{
-    GMainLoop *mainloop = (GMainLoop *) user_data;
-    g_main_loop_quit(mainloop);
-    return FALSE;
-}
-
 static void
 t_testbed_uevent_libudev(UMockdevTestbedFixture * fixture, UNUSED_DATA)
 {
@@ -642,12 +614,16 @@ t_testbed_uevent_libudev(UMockdevTestbedFixture * fixture, UNUSED_DATA)
     g_assert(device != NULL);
     g_assert_cmpstr(udev_device_get_syspath(device), ==, syspath);
     g_assert_cmpstr(udev_device_get_action(device), ==, "add");
+    g_assert_cmpstr(udev_device_get_sysattr_value(device, "idVendor"), ==, "0815");
+    g_assert_cmpstr(udev_device_get_property_value(device, "ID_INPUT"), ==, "1");
     udev_device_unref(device);
 
     device = udev_monitor_receive_device(kernel_mon);
     g_assert(device != NULL);
     g_assert_cmpstr(udev_device_get_syspath(device), ==, syspath);
     g_assert_cmpstr(udev_device_get_action(device), ==, "add");
+    g_assert_cmpstr(udev_device_get_sysattr_value(device, "idVendor"), ==, "0815");
+    g_assert_cmpstr(udev_device_get_property_value(device, "ID_INPUT"), ==, "1");
     udev_device_unref(device);
 
     udev_monitor_unref(udev_mon);
@@ -714,6 +690,41 @@ t_testbed_uevent_libudev_filter(UMockdevTestbedFixture * fixture, UNUSED_DATA)
     udev_unref(udev);
 }
 
+struct event_counter {
+    unsigned add;
+    unsigned remove;
+    unsigned change;
+    gchar last_device[1024];
+};
+
+static void
+on_uevent(UNUSED GUdevClient *client, const gchar *action, GUdevDevice *device, gpointer user_data)
+{
+    struct event_counter *counter = (struct event_counter *)user_data;
+
+    g_debug("on_uevent action %s device %s", action, g_udev_device_get_sysfs_path(device));
+
+    if (strcmp(action, "add") == 0)
+	counter->add++;
+    else if (strcmp(action, "remove") == 0)
+	counter->remove++;
+    else if (strcmp(action, "change") == 0)
+	counter->change++;
+    else
+	g_assert_not_reached();
+
+    strncpy(counter->last_device, g_udev_device_get_sysfs_path(device), sizeof(counter->last_device) - 1);
+}
+
+static gboolean
+on_timeout(gpointer user_data)
+{
+    GMainLoop *mainloop = (GMainLoop *) user_data;
+    g_main_loop_quit(mainloop);
+    return FALSE;
+}
+
+
 static void
 t_testbed_uevent_gudev(UMockdevTestbedFixture * fixture, UNUSED_DATA)
 {
@@ -1116,32 +1127,28 @@ t_testbed_libc(UMockdevTestbedFixture * fixture, UNUSED_DATA)
 
     /* stat */
     g_assert_cmpint(stat ("/sys/bus/pci/devices", &st), ==, 0);
-    g_assert_cmpint(st.st_nlink, ==, 2);
+    g_assert_cmpint(st.st_nlink, >=, 1);
+    g_assert_cmpint(st.st_nlink, <=, 2);
     g_assert_cmpuint(st.st_uid, ==, uid);
     g_assert(S_ISDIR(st.st_mode));
 
     g_assert_cmpint(lstat ("/sys/bus/pci/devices/dev1", &st), ==, 0);
-    g_assert_cmpint(st.st_nlink, ==, 1);
     g_assert_cmpuint(st.st_uid, ==, uid);
     g_assert(S_ISLNK(st.st_mode));
 
     g_assert_cmpint(stat ("/sys/bus/pci/devices/dev1", &st), ==, 0);
-    g_assert_cmpint(st.st_nlink, ==, 2);
     g_assert_cmpuint(st.st_uid, ==, uid);
     g_assert(S_ISDIR(st.st_mode));
 
     g_assert_cmpint(fstatat (AT_FDCWD, "/sys/bus/pci/devices", &st, 0), ==, 0);
-    g_assert_cmpint(st.st_nlink, ==, 2);
     g_assert_cmpuint(st.st_uid, ==, uid);
     g_assert(S_ISDIR(st.st_mode));
 
     g_assert_cmpint(fstatat (AT_FDCWD, "/sys/bus/pci/devices/dev1", &st, AT_SYMLINK_NOFOLLOW), ==, 0);
-    g_assert_cmpint(st.st_nlink, ==, 1);
     g_assert_cmpuint(st.st_uid, ==, uid);
     g_assert(S_ISLNK(st.st_mode));
 
     g_assert_cmpint(fstatat (AT_FDCWD, "/sys/bus/pci/devices/dev1", &st, 0), ==, 0);
-    g_assert_cmpint(st.st_nlink, ==, 2);
     g_assert_cmpuint(st.st_uid, ==, uid);
     g_assert(S_ISDIR(st.st_mode));
 
@@ -1149,17 +1156,16 @@ t_testbed_libc(UMockdevTestbedFixture * fixture, UNUSED_DATA)
     /* statx */
     struct statx stx;
     g_assert_cmpint(statx (AT_FDCWD, "/sys/bus/pci/devices", 0, STATX_TYPE|STATX_NLINK|STATX_UID, &stx), ==, 0);
-    g_assert_cmpint(stx.stx_nlink, ==, 2);
+    g_assert_cmpint(stx.stx_nlink, >=, 1);
+    g_assert_cmpint(stx.stx_nlink, <=, 2);
     g_assert_cmpuint(stx.stx_uid, ==, uid);
     g_assert(S_ISDIR(stx.stx_mode));
 
     g_assert_cmpint(statx (AT_FDCWD, "/sys/bus/pci/devices/dev1", AT_SYMLINK_NOFOLLOW, STATX_TYPE|STATX_NLINK|STATX_UID, &stx), ==, 0);
-    g_assert_cmpint(stx.stx_nlink, ==, 1);
     g_assert_cmpuint(stx.stx_uid, ==, uid);
     g_assert(S_ISLNK(stx.stx_mode));
 
     g_assert_cmpint(statx (AT_FDCWD, "/sys/bus/pci/devices/dev1", 0, STATX_TYPE|STATX_NLINK|STATX_UID, &stx), ==, 0);
-    g_assert_cmpint(stx.stx_nlink, ==, 2);
     g_assert_cmpuint(stx.stx_uid, ==, uid);
     g_assert(S_ISDIR(stx.stx_mode));
 #endif
